/**
 *  Written by Tyler Montbriand in 2004
 *  100% freeware, do whatever the hell you want with it
 *
 *  PSP Port by davi@optixx 2006
*/

#include <stdlib.h>
#include <SDL/SDL.h>
#include <sidplay/sidtune.h>
#include <sidplay/emucfg.h>
#include "sdlext.h"
#include <pspdebug.h>
#include <pspctrl.h>

#define printf  pspDebugScreenPrintf

emuEngine engine;

void sid_callback (void *userdata, Uint8 * stream, int len);

char filename[] = "Chordian.sid";

extern "C"
{
	int SDL_main (int argc, char *argv[]);
}


int
SDL_main (int argc, char **argv)
{
    SceCtrlData pad;
    sceCtrlSetSamplingCycle(0);
    sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
    sidTune tune (filename);
	emuEngine engine;
	AudioSpec spec (44100, AUDIO_S16, 1, 1024, sid_callback, &tune);
	SDL_emucfg cfg (spec);
	printf ("--------------------------------------------------\n");
	printf ("SDL Sidplay by Optixx 2006\n");
    printf ("--------------------------------------------------\n");
	
	if (!tune.open (filename)) {
		printf ("Couldn't find file %s\n", argv[1]);
		return (1);
	}

	struct sidTuneInfo mySidInfo;
	tune.getInfo (mySidInfo);
	if (!tune) {
		fprintf (stderr, "ERROR: %s\n", mySidInfo.statusString);
		exit (-1);
	}
	else {
		printf ("File format  : %s\n", mySidInfo.formatString);
		printf ("Filenames    : ");
		if (mySidInfo.dataFileName)
			printf ("%s", mySidInfo.dataFileName);
		else
			printf ("(None)");
		printf (", ");
		if (mySidInfo.infoFileName)
			printf ("%s", mySidInfo.infoFileName);
		else
			printf ("(None)");
		printf ("\n");
		printf ("Condition    : %s\n", mySidInfo.statusString);
		printf ("--------------------------------------------------\n");
		if (mySidInfo.numberOfInfoStrings == 3) {
			printf ("Name         : %s\n", mySidInfo.nameString);
			printf ("Author       : %s\n",
				mySidInfo.authorString);
			printf ("Copyright    : %s\n",
				mySidInfo.copyrightString);
		}
		else {
			for (int infoi = 0;
			     infoi < mySidInfo.numberOfInfoStrings; infoi++)
				printf ("Description  : %s\n",
					mySidInfo.infoString[infoi]);
		}
		printf ("--------------------------------------------------\n");
		printf ("Load address : $%04x\n", mySidInfo.loadAddr);
		printf ("Init address : $%04x\n", mySidInfo.initAddr);
		printf ("Play address : $%04x\n", mySidInfo.playAddr);
		printf ("--------------------------------------------------\n");
	
    }
	fprintf (stderr,"SDL_Init()\n");

	if (SDL_Init (SDL_INIT_AUDIO | SDL_INIT_VIDEO) < 0) {
		printf ("Couldn't init SDL: %s\n", SDL_GetError ());
		return (1);
	}
	atexit (SDL_Quit);
	fprintf (stderr,"SDL_OpenAudio()\n");
	if (SDL_OpenAudio (&spec, NULL) < 0) {
		fprintf (stderr,"Unable to open audio: %s\n", SDL_GetError ());
		return (1);
	}

	if (!engine.setConfig (cfg)) {
		SDL_CloseAudio ();
		fprintf (stderr,"Couldn't set emulation settings:\n");
		cfg.PrintSettings ();
		return (1);
	}
	fprintf (stderr,"sidEmuInitializeSong\n");
	if (!sidEmuInitializeSong (engine, tune, 0)) {
		fprintf (stderr,"Couldn't initialize song!\n");
		SDL_CloseAudio ();
		return (1);
	}
	fprintf (stderr,"SDL_PauseAudio(0)\n");
	SDL_PauseAudio (0);
	
    while(1){
        sceCtrlReadBufferPositive(&pad, 1);
        if(pad.Buttons & PSP_CTRL_CROSS) {
            break;
        }
	    SDL_Delay (100);
    }
	SDL_PauseAudio (1);
	fprintf (stderr,"SDL_CloseAudi()\n");
	SDL_CloseAudio ();
	return (0);
}

void
sid_callback (void *userdata, Uint8 * stream, int len)
{
	sidTune *tune = (sidTune *) userdata;
	sidEmuFillBuffer (engine, (*tune), stream, len);
}
